//************************************************************************
// Circle.c
// 
// Bresenham -> 14 times faster than using sin and cos
// 1.1.2014
// Peter Menges
//
//
// Compiler WinAVR 3.3
//************************************************************************

#include <avr/io.h>
#include <stdlib.h>
#include <math.h>

#include "protos.h"


//
// Draws a circle at xpos,ypos with Radius radius
//
void Circle(unsigned int xpos, unsigned int ypos, unsigned int radius, unsigned char color)

{
  long x = radius, y = 0;
  long radiusError = 1-x;
 
  while(x >= y)
  {
    SetPixel(x + xpos, y + ypos,color); // lower right
    SetPixel(y + xpos, x + ypos,color); // 
    SetPixel(-x + xpos, y + ypos,color); // lower left
    SetPixel(-y + xpos, x + ypos,color);

    SetPixel(-x + xpos, -y + ypos,color); // upper left
    SetPixel(-y + xpos, -x + ypos,color);
    SetPixel(x + xpos, -y + ypos,color); // upper right
    SetPixel(y + xpos, -x + ypos,color);
 
    y++;
	if(radiusError<0)
		radiusError+=2*y+1;
	else {
        
		x--;
		radiusError+=2*(y-x+1);
		}

  } // while
}

//
// Draws a filled circle at xpos,ypos with Radius radius
//
void FillCircle(unsigned int xpos, unsigned int ypos, unsigned int radius, unsigned char color)
{

  long x = radius, y = 0;
  long radiusError = 1-x;
 
  while(x >= y)
  {
    
		DrawXLine(-x + xpos, x + xpos, y + ypos, color);
		DrawXLine(-y + xpos, y + xpos, x + ypos, color);
		DrawXLine(-x + xpos, x + xpos, -y + ypos,color);	
		DrawXLine(-y + xpos,y + xpos, -x + ypos ,color);
 
	

    y++;
	if(radiusError<0)
		radiusError+=2*y+1;
	else {
        
		x--;
		radiusError+=2*(y-x+1);
		}

  } // while


}
