//************************************************************************
// Draw.C
//
// Linien zeichnen, Strichbreite immer 1
// Rechtecke zeichnen und gefllte Flchen
// 
// Der Nullpunkt der Anzeige ist links oben
//
// hk@holger-klabunde.de
// http://www.holger-klabunde.de
// 08.20.2003
// Compiler AVR-GCC 3.2
// 1.1.2014
// color variable eingefgt um auch Linien lschen zu  knnen
// Peter Menges
//
//************************************************************************
#include <stdlib.h>
#include "protos.h"


void DrawLine(int8_t xstart, int8_t ystart, int8_t xend ,int8_t yend, unsigned char color)
/************************************************************************
 * Bresenham-Algorithmus: Linien auf Rastergerten zeichnen
 *
 * Eingabeparameter:
 *    int xstart, ystart     = Koordinaten des Startpunkts
 *    int xend, yend         = Koordinaten des Endpunkts
 *
 * Ausgabeparameter:
 *    int *npix                 = Anzahl der Pixel
 *    int (*xpix)[i],(*ypix)[i] = Koordinaten des i-ten Pixels
 *
 ***********************************************************************/
{
  long x, y, t, dist, xerr, yerr, dx, dy, incx, incy;

/* Entfernung in beiden Dimensionen berechnen */
     dx = xend - xstart;
     dy = yend - ystart;

/* Vorzeichen des Inkrements bestimmen */
     if(dx<0)
     {
       incx = -1;
       dx = -dx;
     }
     else
       incx = dx ? 1 : 0;

     if(dy < 0)
     {
       incy = -1;
       dy = -dy;
     }
     else
       incy = dy ? 1 : 0;

//** feststellen, welche Entfernung grer ist
     dist = (dx > dy)?dx:dy;

//** Initialisierungen vor Schleifenbeginn
     x = xstart;
     y = ystart;
     xerr = yerr = (dist) >> 1; //dx;
     // yerr = dist >> 1; //dy1;

//** Pixel berechnen
     for(t = 0; t < dist; ++t)
     {
       SetPixel(x, y,color);

       xerr += dx;
       yerr += dy;

       if(xerr >= dist)
       {
         xerr -= dist;
         x += incx;
       }

       if(yerr >= dist)
       {
         yerr -= dist;
         y += incy;
       }
     }

     SetPixel(xend, yend,color);
} // DrawLine

//
// Zeichnet eine horizontale Linie absolut von X1 bis X2 auf Zeile Y
//
void DrawXLine(unsigned int x1, unsigned int x2, unsigned int y, unsigned char color)
{
 unsigned int i,xstart,xend;

 if(x2>x1) //Immer von links nach rechts zeichnen
  { xstart=x1; xend=x2; }
 else
  { xstart=x2; xend=x1; }
 
  
 for(i=xstart; i<=xend; i++)
  {
   SetPixel(i,y,color);
  }
}

//
// Zeichnet eine vertikale Linie absolut von Y1 bis Y2 auf Spalte X
//
void DrawYLine(unsigned int y1, unsigned int y2, unsigned int x, unsigned char color)
{
 unsigned int i,ystart,yend;

 if(y2>y1)
  { ystart=y1; yend=y2; }
 else
  { ystart=y2; yend=y1; }

 for(i=ystart; i<=yend; i++)
  {
   SetPixel(x,i,color);
  }
}

//
// Zeichnet eine horizontale Linie mit Lnge len ab Position X auf Zeile Y
// Von links nach rechts
//
void DrawXLineLen(unsigned int x, unsigned int len, unsigned int y, unsigned char color)
{ DrawXLine(x,x+len-1,y, color); 

}

//
// Zeichnet eine vertikale Linie mit Lnge len ab Position Y auf Spalte X 
// Von oben nach unten
//
void DrawYLineLen(unsigned int y, unsigned int len, unsigned int x, unsigned char color)
{ DrawYLine(y,y+len-1,x,color); 

}

//
// Zeichnet ein Rechteck mit Breite width, Hhe height ab Position X,Y 
//
void DrawRect(unsigned int x, unsigned int y, unsigned int width, unsigned int height, unsigned char color)
{
 DrawXLineLen(x,width,y,color);
 DrawXLineLen(x,width,y+height-1,color);
 DrawYLineLen(y,height,x,color);
 DrawYLineLen(y,height,x+width-1,color);
}

//
// Fllt ein Rechteck mit Breite width, Hhe height ab Position X,Y 
//
void FillRect(unsigned int x, unsigned int y, unsigned int width, unsigned int height, unsigned char color)
{
 unsigned int i;

 for(i=y; i<(y+height); i++) DrawXLineLen(x,width,i,color);
}
