//#################################################################
// fontsoft.c
//
// Mods by Peter Menges: (Dez. / 2013)
// 
// Fontbehandlung und Fontaufbau gendert
//
// Fontaufbau: Fonts sind immer 8 mal n aufgebaut
// D.h, immer 8-Bit breit und n-Bit hoch
// Es wird von D7 nach D0 gelesen.
// Also von Links nach rechts. 
// Das erste Byte eines Fonts ist immer die oberste (Pixel) Zeile
// und D7 das erste Bit links.
// Die nachfolgenden Bytes entsprechen der Hhe
// von oben nach unten gelesen.
// Grund der nderung: Mehr Fonts mit diesem Aufbau verfgbar.
//
// Basic coding by:
// hk@holger-klabunde.de
// http://www.holger-klabunde.de
// 08.10.2003
// Compiler AVR-GCC 3.2
//#################################################################
#include <avr/io.h>
#include <avr/pgmspace.h>
#include "protos.h"

unsigned char fontwidth=0,fontheight=0;   //Breite,Hhe in Pixeln
unsigned char *font;
// Namensgebung der Fonts: Name_f hhe in pixel
// Breite immer 8 Bits (16 Bits und mehr: fehlt)!
extern prog_uchar hercules_f16[];
extern prog_uchar hercules_f08[];
extern prog_uchar hercital_f08[];
extern prog_uchar thinasci_f07[];


//
// Setzt den Font
//
void SetFont(unsigned char num)

{

	switch(num){
	case 1:	font = hercules_f16;
			fontwidth=8;
			fontheight=16;
			break;
	case 2: font = hercules_f08;
			fontwidth=8;
			fontheight=8;
			break;
	case 3: font = hercital_f08;
			fontwidth=8;
			fontheight=8;
			break;
	case 4: font = thinasci_f07;
			fontwidth=8;
			fontheight=7;
			break;
	default:font = hercules_f08;		
			fontwidth=8;
			fontheight=8;;
			break;
	}

}

//
//Schreibt ein einzelnes Zeichen an Position x,y
// 25.12.2013 Me: Grundstzliche nderung:
//				  Es wird Zeilenweise geschrieben, von
//                oben nach unten. 
//
void LCDSoftChar(unsigned char ch, unsigned int xpos, unsigned int ypos, unsigned char color)
{
unsigned char by,mask,v;
unsigned char i,j;

 if(fontwidth==0) return; //Kein Font ausgewhlt
 
 for(i=0; i<fontheight; i++) //ber die Breite des Zeichens
  {
   
     by=pgm_read_byte(&font[ch*fontheight+i]); //Ein Byte des Zeichens holen
     mask=0x80;               //Links nach rechts schreiben
     for(j=0; j<8; j++)
      {
	  	v = (by&mask);

		if ( !color){

				if ( v )
					v = 0;
				else
					v = 1;
		}
				

	  	
       SetPixel(xpos+j,ypos+i,v);

       mask >>=1; //Nchstes Bit
      }//for j 
   }//for i
}

//
//Schreibt einen Text an Position x,y
//Mit automatischem Zeilenvorschub falls der Text zu lang war.
//Noch keine Steuerzeichen wie TAB,CR,LF integriert
//
void LCDSoftText(char *__putstr, unsigned int xpos, unsigned int ypos, unsigned char color)
{
 char __ps;
 unsigned int xp,yp;

 if(fontwidth==0) return; //Kein Font ausgewhlt

 xp=xpos; yp=ypos;
 
 while((__ps = *__putstr))
  {
   __putstr++;
   if (__ps== 0) break;
   LCDSoftChar(__ps,xp,yp,color);
   xp+=fontwidth; //Nchstes Zeichen
   if(xp>=LCD_WIDTH) { xp=0; yp+=fontheight; } //Nchste Zeile
  }
}

//
//Zeigt eine bersicht der im Font enthaltenen Zeichen
//
void ShowSoftFont(unsigned char num, unsigned char color)
{
 unsigned int i,xp,yp;
 
 ClearScreen();

 xp=0; yp=0;
 SetFont(num);
 
 for(i=0; i<255; i++)
  {
   LCDSoftChar(i,xp,yp,color);
   xp+=fontwidth; //Nchstes Zeichen
   if(xp>=LCD_WIDTH) { xp=0; yp+=fontheight; } //Nchste Zeile
  }
}

//
//Zeigt eine bersicht der im Font enthaltenen Zeichen
//
void ShowSoftFontV(unsigned char num, unsigned char color)
{
 unsigned int i,xp,yp;
 
 ClearScreen();

 xp=0; yp=0;
 SetFont(num);
 
 for(i=0; i<255; i++)
  {
   LCDSoftCharV(i,xp,yp,color);
   yp-=fontwidth; //Nchstes Zeichen
   if(yp>=LCD_HEIGHT) { yp=LCD_HEIGHT-1; xp+=fontheight; } //Nchste Zeile
  }
}

//
//Schreibt einen Text 90 nach links gekippt an Position x,y
//Mit automatischem Zeilenvorschub falls der Text zu lang war.
//Noch keine Steuerzeichen wie TAB,CR,LF integriert
//
void LCDSoftTextV(char *__putstr, unsigned int xpos, unsigned int ypos, unsigned char color)
{
 char __ps;
 unsigned int xp,yp;

 if(fontwidth==0) return; 								//Kein Font ausgewhlt

 xp=xpos; yp=ypos;
 
 while((__ps = *__putstr))
  {
   __putstr++;
   if (__ps== 0) break;
   LCDSoftCharV(__ps,xp,yp,color);
   yp-=fontwidth; 											//Nchstes Zeichen
   if(yp>=LCD_HEIGHT) { yp=LCD_HEIGHT-1; xp+=fontheight; } //Nchste Zeile
  }
}

//
//Schreibt ein einzelnes Zeichen 90 nach links gekippt an Position x,y
// Von unten (y) nach (y-) oben
//
void LCDSoftCharV(unsigned char ch, unsigned int xpos, unsigned int ypos, unsigned char color)
{
unsigned char by,mask, v;
unsigned char i,j;
 
 for(i=0; i<fontheight; i++) //ber die Breite des Zeichens
  {
   
     by=pgm_read_byte(&font[ch*fontheight+i]); //Ein Byte des Zeichens holen
     mask=0x80;               //Links nach rechts schreiben
     for(j=0; j<8; j++)
      {
		v = (by&mask);

		if ( !color){

				if ( v )
					v = 0;
				else
					v = 1;
		}      
	   SetPixel(xpos+i,ypos-j,v);

       mask >>=1; //Nchstes Bit
      }//for j 
   }//for i
}

