//************************************************************************
// KS108.C
// LowLevel Routinen fr LCD-Displays mit KS0108
//
// Hier fr Displaytech 64240A
//
// Mit bis zu vier KS108 Chips also bis 240x64
// Ausgelegt bisher nur auf diesen Displaytyp
//
// Der Nullpunkt des Displays liegt oben,links. Fr unten rechts
// mssen dann zustzliche Umrechnungen stattfinden.
//
// hk@holger-klabunde.de
// http://www.holger-klabunde.de
// 11.10.2003
// Compiler WinAVR
//
// MODS by Peter Menges (PME) 2013: 
// To speed up the performance of the output to the display
// SRAM of size 1k is allocated. All modifications (drawing something) is
// made internal and the display is updated in one shot. 
// This consumes much SRAM, but the performance is more than twice of the 
// original code.
// Be aware that the timing is coded for 1 MHz clock.
// See the notes (prefixed with PME ) I made at several routines. To port for higher clock rates
// only some minor modifications are necessary.
// 
// Thanks to Holger for his basic coding!
//
// Coding is for ATMega32 running at 1 MHz from internal clocksource.
//************************************************************************
#include <avr/io.h>
#include <avr/interrupt.h> 
#include <util/delay.h>
#include <stdio.h>
#include "protos.h"
#include "ks108.h"
//#include "uart.h"

unsigned char Dis_Buf[1024]={0}; /* PME 128 x 64 Pixel / 8 = 1024 */
extern unsigned char fontheight;

//volatile int disp_update = 0;

//
// Read a byte from the  display memory
//
/****************** NEVER USED HERE *********************
unsigned char ReadDisplay(void)
{
 unsigned char by;

 
 RS_ON();
 RW_ON(); //Wir wollen lesen

//Display mu zweimal gelesen werden !
//Dummy Read
E_ON();
E_OFF();

 DATA_DIR_IN();

 RS_ON();
 RW_ON(); //Wir wollen lesen

 E_ON();
 by=READ_DATA(); //DATA RAM lesen
 E_OFF();

 RW_OFF(); //zurck auf schreiben

 DATA_DIR_OUT();
 
 return by;
}
*****************************************************/

	
//
//Write a byte to the display
//Register Select must selected before!
//
void WriteDisplay(unsigned char dat)
{
 DATA_DIR_OUT();

 RW_OFF(); 		//Display schreiben setzen

 WRITE_DATA(dat);
 E_ON();
 E_OFF();

}

//
//Set a Pixel within the RAM-Buffer
//val=0; Pixel clear
//val=1; Pixel set
//
unsigned char SetPixel(unsigned int x, unsigned int y, unsigned char val )
{
 unsigned char *p, tmp;
 int idx;
  
 
 
 if(x >= LCD_WIDTH) return 1; //Out of bounds
 if(y >= LCD_HEIGHT) return 1;


 
	idx = 0;
	
	idx = y/8;
			
	idx = idx * 64;
	
		if ( x <= 63 )
				idx += x; 
		else 
				idx += (512 + ( x - 64));
				
	if ( idx > 1023 || idx < 0){
				return 1;
	}
							

	p = &Dis_Buf[ idx ];
	tmp = y % 8;
	
	if(val)
	*p |= ( 1 << tmp);		
	else 
	*p &= ~( 1 << tmp); 

	 return 0;
}

//
//Calculate the chipnumber from the x pos.
//
unsigned char GetChipNumber(unsigned int xpos)
{
 return xpos/64; //Jeder Chip kontrolliert 64x64 Pixel
}

//
//Set Chipnumber
//
void SetChipNumber(unsigned char num)
{
 E_OFF();
 CS1_OFF();
 CS2_OFF();
 switch (num){
	case 0: CS1_ON(); break;
	case 1: CS2_ON(); break;
	default: break;
 }
 
}

//
//Calculate the Page from the y pos.
//
unsigned char GetPageNumber(unsigned char ypos)
{
 return ypos/8; //Jede Page ist 8 Zeilen hoch
}

//
//Set Pagennumber of the chip selected
//
unsigned char SetPageNumber(unsigned char num)
{
 
 RS_OFF(); 					//Instruction setzen
 WriteDisplay(0xB8|num); 	//Pagenummer setzen

 RS_ON(); 					//Zurck auf Daten
 return 0;
}
//
// INIT IO
//
void InitDisplay(void)
{
	INIT_CONTROL_PINS();
	RSTB_OFF(); 			// RESET
	_delay_us(10);
	RSTB_ON(); 				// Go
	_delay_us(10);
	DisplayOn();
	
}

//
//Each KS0108 needs the DisplayOn command
//
void DisplayOn(void)
{
 unsigned char i;

 E_OFF();

 for(i=0; i< MAX_CHIPS; i++)
  {
   SetChipNumber(i); 
    RS_OFF(); 				//Prepare for instruction
   WriteDisplay(0x3F); 		//Set instruction display on
   RS_ON(); 				//Switch back to data
  } 

}
//
// Disable displaying the content of the display ram
//
void DisplayOff(void)
{
 unsigned char i;

  E_OFF();

 for(i=0; i< MAX_CHIPS; i++)
  {
   SetChipNumber(i); 
    RS_OFF(); 			//prepare for instruction 
   WriteDisplay(0x3E); 	//Set instruction display off
   RS_ON();				//Switch back to data
  } 

}

//
// Clear the RAM-Buffer
// PME Sure that this could be done much simpler ;-)
// But avoiding libraries saves flash ram
//
void ClearScreen(void)
{ 

 unsigned int i;
 
 for(i=0; i < 1024; i++ ) Dis_Buf[i] = 0x00;
 
 
}

//
// mode = 1 Display_Off during update 
// other value: don't 
// without DisplayOff, the display does not flicker, when
// high refresh rates are required
// PME 1.1.2014
//

void UpdateDisplay(uint8_t mode)
{
	unsigned char j,i;
	char k;
	unsigned char  *p;
	
	
	
	if (mode)
	DisplayOff(); // PME Do not show the update. Doesn't work with all displays

	k= 0; 
	p = Dis_Buf;
	
	
	 do
	  {
	   SetChipNumber(k); 
	
	   for(j=0; j<8; j++) // PME Run down the "video" memory and write it to the display
		{
		 SetPageNumber(j);
		 RS_OFF(); 			// Do not forget to: Set Instruction 13.4.2007 15:35 Uhr
		 WriteDisplay(0x40); //Set Column 0 
         RS_ON();
		 RW_OFF(); 			//Set display to write operation
	
		 
		 for(i=0; i<64; i++)
		  {  

		  	WRITE_DATA(*p++);
						
			E_ON();			// Increment the adress counter in the display
			E_OFF();
			 
		  }
		}
	
	  k++;
	 }while(k < MAX_CHIPS);
 

 if (mode )
 DisplayOn(); // Now show the result
	
}
// Scrolls the screen 1 Line of the selected font up
// One line is 64 bytes wide
// The height has 8 rows of byte cells
// Each chip controls 64x64 pixels
// From left to right the chipnumbers are 0,1
// Within the Dis_Buf memory each 512 byte block 
// represents one chip starting with chip 0
//
// This version supports only 2 KS0108!
//
#define CHIPSIZE	512

void ScrollUp(void)
{

	int i,z, step;
	unsigned char *s, *d;


	if ( fontheight == 0 ) 
			return;

	step = fontheight / 8;

	if ( fontheight % 8) 		// Font is not by 8
			step++;			  // Line more to scroll	

								// calculate the distance

	i = step * 64;		//one row has 64 bytes
	
	// move for chip 0
	s = &Dis_Buf[i];  	// source
	d = &Dis_Buf[0];	// destination
	for(z=i; z < CHIPSIZE; z++)
		*d++ = *s++;

	// move for chip 1
	s = &Dis_Buf[CHIPSIZE + i ];	// source
	d = &Dis_Buf[CHIPSIZE];			// destination

	for(z=CHIPSIZE+i; z < (CHIPSIZE*2); z++ )
		*d++ = *s++;

	// Now clear the tails
	// Chip 0

	d = &Dis_Buf[CHIPSIZE - i];
	s = &Dis_Buf[CHIPSIZE];
	for(; d < s; )
		*d++ = 0x00;

	// Chip 1

	d = &Dis_Buf[CHIPSIZE*2 - i];
	s = &Dis_Buf[CHIPSIZE*2];
	for(; d < s; )
		*d++ = 0x00;

}


